/*******************************************************************************
* ADECRYPT.C
*
* Name         : AGI Decryptor
* Description  : Decrypts the file AGI into AGI.EXE
*                Written in standard C to be very portable, though since it only
*                decrypts MS-DOS interpreters, porting it to any system other
*                than MS-DOS or Windows is not very important :)
* Author       : By Brian Provinciano
* Created      : March 3rd, 2000
* Modified     : March 4th, 2000
* Version      : 1.0.0.5  (Major Version.Minor Version.Release.Build)
*******************************************************************************/

#include"adecrypt.h"

void main(int argc, char **argv)
{
/* Print an about message */
	printf("AGI Decryptor\n"
          "Version 1.0.0.5\n"
          "By Brian Provinciano\n\n");

   	strcpy(LOADER_FILE_NAME, (argc < 2)? "SIERRA.COM":argv[argc-1]);

   	LoadEncryptionString();
   	LoadEncryptedData();
	DecryptAGI();

   	printf("\nSuccess!\n");
}

/*******************************************************************************
* LoadEncryptionString
* Loads the encryption string into memory
*******************************************************************************/
void LoadEncryptionString()
{
/* Open the loader to obtain the decryption string */
	if((SIERRA_COM = fopen(LOADER_FILE_NAME, "rb")) == NULL)
		ErrorMessage("Unable to open loader file for reading!");

   printf("Loading Encryption String From %s...\n", LOADER_FILE_NAME);

/* Do a loop to find where the encryption string begins */
	KeyFound = FALSE;

	while(!feof(SIERRA_COM) && !KeyFound) {
		if(
			fgetc(SIERRA_COM) == 'k' &&
			fgetc(SIERRA_COM) == 'e' &&
			fgetc(SIERRA_COM) == 'y' &&
			fgetc(SIERRA_COM) == 'O' &&
			fgetc(SIERRA_COM) == 'f' &&
			fgetc(SIERRA_COM) == 's')
				KeyFound = TRUE;
	};

/* If keyOfs is not found, it must not be an AGI loader so
   decryption cannot occur */
	if(!KeyFound) ErrorMessage("Decryption String not found!");

/* Seek to the offset of the encrypition string and load it into memory */
	fseek(SIERRA_COM, 2, SEEK_CUR);
	fread(DecryptionString, 128, 1, SIERRA_COM);

/* Test to see if AGI is encrypted */
	TempPointer = 0;
	ENCRYPTED = FALSE;

	do {
		if(DecryptionString[TempPointer] != 'k')
		ENCRYPTED = TRUE;

		TempPointer++;
	} while(TempPointer < 128 && !ENCRYPTED);

   	if(!ENCRYPTED)
   		ErrorMessage("File AGI Not Encrypted!");

/* Close the loader, it is no longer needed */
	fclose(SIERRA_COM);
}

/*******************************************************************************
* LoadEncryptedData
* Loads the data of the file AGI into memory.
*******************************************************************************/
void LoadEncryptedData()
{
/* Open the loader to obtain the executable data */
	if((AGI = fopen("AGI", "rb")) == NULL)
		ErrorMessage("Unable to open file AGI for reading!");

   	printf("Loading Execrypted Data From AGI...\n");

/* Get the file's length */
	fseek(AGI, 0, SEEK_END);

	AGI_FILE_LENGTH = (unsigned) ftell(AGI);

/* Check if the file length is valid */
	if(AGI_FILE_LENGTH < 1)
		ErrorMessage("Invalid AGI File Size!");

/* Allocate memory for the file data */
	if((EncryptedExeData = (byte *) malloc(AGI_FILE_LENGTH)) == NULL)
		ErrorMessage("Unable to allocate memory!");

/* Copy the file data to the memory */
	fseek(AGI, 0, SEEK_SET);
	fread(EncryptedExeData, AGI_FILE_LENGTH, 1, AGI);

/* Close the file, it is no longer needed */
	fclose(AGI);
}

/*******************************************************************************
* DecryptAGI
* Decrypts AGI into AGI.EXE
*******************************************************************************/
void DecryptAGI()
{
   	printf("Decrypting To AGI.EXE...\n");

/* Allocate the memory for decrypting the data */
	if((DecryptedExeData = (byte *) malloc(AGI_FILE_LENGTH)) == NULL)
		ErrorMessage("Unable to allocate memory!");

/* Begin decryption */
	ExeDataPointer = 0;

	while(ExeDataPointer < AGI_FILE_LENGTH) {
		TempPointer = 0;

/* XOR 128 bytes of the file data with the encryption string */
		while(TempPointer < 128 && ExeDataPointer < AGI_FILE_LENGTH) {
			DecryptedExeData[ExeDataPointer] = (byte)
				(EncryptedExeData[ExeDataPointer] ^ DecryptionString[TempPointer]);

			TempPointer++;
			ExeDataPointer++;
		};

/* Shift the entire encryption string one bit to the right */
		LastCarryFlag = (byte) (DecryptionString[127] & 1);

		for(TempPointer = 0; TempPointer < 128; TempPointer++) {
			NewCarryFlag = (byte) (DecryptionString[TempPointer] & 1);

			DecryptionString[TempPointer] >>= 1;
			DecryptionString[TempPointer] |= (byte)(OldCarryFlag <<= 7);

			OldCarryFlag = NewCarryFlag;
		};

		DecryptionString[0] |= (byte) (LastCarryFlag <<= 7);
	}

/* Open the loader to write the unencrypted data */
   if((AGI_EXE = fopen("AGI.EXE", "wb")) == NULL)
		ErrorMessage("Unable to open file AGI.EXE for writing!");

/* Write the unencrypted data to the file */
   fwrite(DecryptedExeData, AGI_FILE_LENGTH, 1, AGI_EXE);

/* Close the file, it is no longer needed */
   fclose(AGI_EXE);
}

/*******************************************************************************
* ErrorMessage
* Prints a message to the screen and then exits the program
*******************************************************************************/
void ErrorMessage(char *Message)
{
	printf("\n%s\n", Message);
	exit(1);
}

/******************************************************************************/
